@@ -101,6 +101,10 @@ class EncodeFile:
     def md5sum(self) -> str:
         return self.portal_file["md5sum"]
 
+    @property
+    def submitted_file_name(self) -> str:
+        return self.portal_file["submitted_file_name"]
+
     def get(
         self, key: str, default: Optional[U] = None
     ) -> Union[Optional[T], Union[T, U]]:
@@ -168,7 +172,7 @@ class EncodeFile:
         dataset: str,
         derived_from: List[str],
         file_params: FileParams,
-        file_size: str,
+        file_size: int,
         file_md5sum: str,
         step_run_id: str,
         submitted_file_name: str,
@@ -269,6 +273,7 @@ class EncodeAnalysis:
 class EncodeExperiment:
     INTERNAL_STATUS_KEY = "internal_status"
     INTERNAL_STATUS_POST_ACCESSIONING = "pipeline completed"
+    ALLOWED_REPLICATE_STATUSES = ("released", "in progress")
 
     def __init__(self, portal_experiment: Dict[str, Any]):
         self.at_id = portal_experiment["@id"]
@@ -291,6 +296,7 @@ class EncodeExperiment:
             [
                 rep.get("biological_replicate_number")
                 for rep in self.portal_properties["replicates"]
+                if rep["status"] in self.ALLOWED_REPLICATE_STATUSES
             ]
         )
         return len([rep for rep in bio_reps if rep is not None])
@@ -300,6 +306,7 @@ class EncodeExperiment:
             [
                 rep.get("technical_replicate_number")
                 for rep in self.portal_properties["replicates"]
+                if rep["status"] in self.ALLOWED_REPLICATE_STATUSES
             ]
         )
         return len([rep for rep in tech_reps if rep is not None])
